import unicodedata

import os
import sys

from difflib import get_close_matches

regioninfo = open('geodata/iso3166_2_regions.txt','r')
regionfips = open('geodata/iso3166_2_regions_fips.txt','r')
geonameinfo = open('geodata/admin1Codes.txt','r')

# I included the headers in my export here, so slice them out.
regionlines = regioninfo.readlines()[1:]
regionlinesfips = regionfips.readlines()
geonamelines = geonameinfo.readlines()

fipsmap = {}
geonamemap = {}

failcount = 0

finalmapping = []

def get_country_region_match(word, country, reference):
    # Gets a list of keys (ie region names in the use case)
    region_names = [ item for item in country]
    matches = get_close_matches(word, region_names, 2, 0.6)

    if len(matches) is 1:
        return country[matches[0]]
    elif len(matches) is 0:
        sys.stderr.write("No matches for %s: %s\n" % (reference, word))
    else:
        sys.stderr.write("Match was ambiguous, returned the first option (%s)anyway: %s\n" % ( word, reference) )
        return country[matches[0]]

# Common Data Hub ISO3166-2 with FIPS Codes
for regiondata in regionlinesfips:
    # Split on tabs
    regiondata_list = regiondata.strip().split('\t')
    # Hard coded offset = common data hub primary key
    fipsmap[regiondata_list[6]] = regiondata_list


# Geonames Data, we use this as a fallback if we can't match CDH data.
for regiondata in geonamelines:
    # Split on tabs
    regiondata_list = regiondata.strip().split('\t')

    # Some entries in the file don't play nice with this expectation
    if len(regiondata_list) < 2:
        continue

    # Get a dictionary going.
    try:
         geonamemap[regiondata_list[0].split('.')[0]]
    except:
         geonamemap[regiondata_list[0].split('.')[0]] = {}

    # Normalize UTF8 characters for better tolerance of accents in case we do goofy name matching.
    norm = regiondata_list[1].strip()
    # Country code dictionary, each with sub-elements for every region/state in that country.
    geonamemap[regiondata_list[0].split('.')[0]][norm] = regiondata_list[0].split('.')[1]

# CDH ISO-3166-2 ISO Data
for regiondata in regionlines:
    regiondata_list = regiondata.strip().split('\t')

    country_id = regiondata_list[0].split('-')[0].upper()

    try:
        # Try cross-reference on CDH database
        fipscode = fipsmap[regiondata_list[6]][0].split(' ')[0][2:]
    except:
        try:
            # Try fuzzy matching region names between CDH and Geonames
            norm = regiondata_list[1].strip()
            fipscode = get_country_region_match(norm, geonamemap[country_id], regiondata_list[0])
        except:
            # Nothing worked ;/
            fipscode = ''

    if fipscode:
        finalmapping.append( {
            'fips' : fipscode,
            'iso': regiondata_list[0].split('-')[1],
            'country' : regiondata_list[0].split('-')[0].upper()
        })
    else:
        fipscode = None


result = "\n".join( ["COUNTRY:\t%s\tFIPS:\t%s\tISO:\t%s" % (item['country'], item['fips'], item['iso']) for item in finalmapping])
print result